#!/bin/bash
#
# Given a tarred restart file, puts in on the local repository
# 
# Author: F. Massonnet
# Date  : November 2016
# Update: January 2017, to make assembly of split restarts on the HPC

set -o nounset
set -o errexit
#set -x

if [ $# == 0 ]
then
  echo "dispatch.bash restart-file.tar"
  echo "EXAMPLE: "
  echo "dispatch.bash /esnas/exp/nemo/original_files/a05p/restart_files/a05p/19580101/fc00/restarts/RESTO_a05p_19580101_fc00_32_19890101-19891231.tar"
  echo ""
  exit
fi

rfile=$1

workdir=/scratch/Earth/$USER/TMP_${RANDOM}
workdir=/scratch/Earth/$USER/TMP_16621

mkdir -p $workdir
echo "WORKDIR >>>>>      $workdir      <<<<<<"

if [ ! -f $rfile ]
then
  echo "$rfile not found"
  exit
fi

# Obtain some information...
fbase=`basename $rfile`
exp=$(echo $fbase   | cut -d'_' -f 2)
sdate=$(echo $fbase | cut -d'_' -f 3)
memb=$(echo $fbase  | cut -d'_' -f 4)
chunk=$(echo $fbase | cut -d'_' -f 5)
runperiod=$(echo $fbase | cut -d'_' -f 6)
runperiod=${runperiod%.tar} # remove the .tar
endperiod=$(echo $runperiod | cut -d'-' -f 2)

#tar xf $rfile -C $workdir


# Check whether the restarts are rebuilt or not. If not, stop -- not coded yet
nfiles_oce=`ls $workdir/NEMO_Restart_${chunk}/${exp}_????????_restart_oce*.nc | wc -l`
nfiles_ice=`ls $workdir/NEMO_Restart_${chunk}/${exp}_????????_restart_ice*.nc | wc -l`

if [ $nfiles_oce != 1 ] 
then
  echo "The ocean or the ice restart files are not rebuilt"
  # First we need to get the time step information (the ????????)
  timstep=`ls $workdir/NEMO_Restart_${chunk}/${exp}_????????_restart_oce_0000.nc | awk -F/ '{print $NF}' | cut -d "_" -f 2`
  # Strategy: send all the files to a remote repo on the HPC, assemble them, 
  # and take them back here.
  read -p "Enter your MN3 login (e.g. bsc32XXX): " bscuser
  hpctmpdir="/scratch/tmp/TMP_${RANDOM}"
  echo "HPC temporary dir: $hpctmpdir"
  ssh ${bscuser}@dt01.bsc.es "mkdir -p $hpctmpdir"
  rsync -rlptoDv $workdir/NEMO_Restart_${chunk}/${exp}_${timstep}_restart_oce_????.nc ${bscuser}@dt01.bsc.es:$hpctmpdir

  # Run the rebuild
  ssh ${bscuser}@dt01.bsc.es "/home/bsc32/${bscuser}/rebuild_nemo $hpctmpdir/${exp}_${timstep}_restart_oce $nfiles_oce"

  # Download it back here
  rsync -rlptoDv ${bscuser}@dt01.bsc.es:$hpctmpdir/${exp}_${timstep}_restart_oce.nc $workdir/NEMO_Restart_${chunk}/
fi
if [ $nfiles_ice != 1 ]
then
  echo "The ice restart files are not rebuilt"
  # First we need to get the time step information (the ????????)
  timstep=`ls $workdir/NEMO_Restart_${chunk}/${exp}_????????_restart_ice_0000.nc | awk -F/ '{print $NF}' | cut -d "_" -f 2`
  # Strategy: send all the files to a remote repo on the HPC, assemble them,
  # and take them back here.
  read -p "Enter your MN3 login (e.g. bsc32XXX): " bscuser
  hpctmpdir="/scratch/tmp/TMP_${RANDOM}"
  echo "HPC temporary dir: $hpctmpdir"
  ssh ${bscuser}@dt01.bsc.es "mkdir -p $hpctmpdir"
  rsync -rlptoDv $workdir/NEMO_Restart_${chunk}/${exp}_${timstep}_restart_ice_????.nc ${bscuser}@dt01.bsc.es:$hpctmpdir

  # Run the rebuild
  ssh ${bscuser}@dt01.bsc.es "/home/bsc32/${bscuser}/rebuild_nemo $hpctmpdir/${exp}_${timstep}_restart_ice $nfiles_ice"

  # Download it back here
  rsync -rlptoDv ${bscuser}@dt01.bsc.es:$hpctmpdir/${exp}_${timstep}_restart_ice.nc $workdir/NEMO_Restart_${chunk}/
fi

fileoce=$workdir/NEMO_Restart_${chunk}/${exp}_????????_restart_oce.nc
fileice=$workdir/NEMO_Restart_${chunk}/${exp}_????????_restart_ice.nc

# Get grid information 
nx=`ncdump -h  $fileoce | grep "x =" | awk {'print $3'}`
ny=`ncdump -h  $fileoce | grep "y =" | awk {'print $3'}`
nz=`ncdump -h  $fileoce | grep "z =" | awk {'print $3'}`

if [ $nx = 362 ] && [ $ny = 292 ]
then
  grid=ORCA1
elif [ $nx = 1442 ] && [ $ny = 1050 ]
then
  grid=ORCA025
else
  echo "Not coded yet, to add"
  exit
fi

# Get sea ice model information. Looking for variable a_i_htc1 which is LIM3-indicative
if [ `ncdump -h $fileice | grep a_i_htc1 | wc -l` != 0 ]
then
  seaicemodel=LIM3
else
  echo "sea ice model unknown, or maybe LIM2?"
fi


# Create output directory for ocean and ice
if [ ! -d /esnas/releases/ic/ocean/${grid}L${nz} ]
then
  echo "Configuration does not exist: ${grid}L${nz}"
  echo "There must be an error."
  exit
fi

# Do the ocean
mkdir -p /esnas/releases/ic/ocean/${grid}L${nz}/${exp}

fileout=/esnas/releases/ic/ocean/${grid}L${nz}/${exp}/${exp}_${memb}_${endperiod}_restart.nc
cp $fileoce $fileout
chmod 777 $fileout
gzip -vf $fileout

# Do the ice
mkdir -p /esnas/releases/ic/ice/${grid}_${seaicemodel}/${exp}
fileout=/esnas/releases/ic/ice/${grid}_${seaicemodel}/${exp}/${exp}_${memb}_${endperiod}_restart_ice.nc
cp $fileice $fileout
chmod 777 $fileout
gzip -vf $fileout




echo ""
echo " The files are ready. Don't forget to upload them on the HPC "
echo " For upload on MN3:"
echo " rsync -rlptoDv /esnas/releases/ic/ocean/${grid}L${nz}/${exp}/${exp}_${memb}_${endperiod}_restart.nc YOURLOGIN@dt01.bsc.es:/gpfs/projects/bsc32/repository/ic/ocean/${grid}L${nz}/${exp}/"
echo " rsync -rlptoDv /esnas/releases/ic/ice/${grid}_${seaicemodel}/${exp}/${exp}_${memb}_${endperiod}_restart_ice.nc YOURLOGIN@dt01.bsc.es://gpfs/projects/bsc32/repository/ic/ice/${grid}_${seaicemodel}/${exp}/"
echo ""



